//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoigdt.cpp,v 1.14 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoigdt.cpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The internal implementation of various Roma gadgets.
//--------------------------------------------------------------------

#include <ctype.h>
#include <kxcointa.hpp>
#include <kxcoigdt.hpp>
#include <kxcoipxy.hpp>
#include <kxcodefs.hpp>
#include <kxcoicxt.hpp>

TRACE_MODULE_VERSION($Revision: 1.14 $);


//====================================================================
// The RomaImplShareable class.
//====================================================================

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplShareable::~RomaImplShareable()
{
}

//--------------------------------------------------------------------
// This method should be called by an owner of the shared object
// instance to increment the reference counter, so the instance will
// not be deleted while it has one or more owners.
//--------------------------------------------------------------------
void RomaImplShareable::lockInstance() 
{ 
    ++m_lockCount; 
}

//--------------------------------------------------------------------
// This method should be called by an owner of the shared object
// instance to decrement the reference counter, so the instance will
// be deleted if there are no more owners referencing it.
//--------------------------------------------------------------------
void RomaImplShareable::releaseInstance() 
{ 
    if (--m_lockCount == 0) 
    {
        delete this; 
    }
}


//====================================================================
// The RomaImplDirAccess class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplDirAccess::RomaImplDirAccess()
{
    TRACE_ENTRY(RomaImplDirAccess::RomaImplDirAccess, << TRACE(this));

    kxcSetDirAccessOptDefault(&m_data);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplDirAccess::~RomaImplDirAccess()
{
    TRACE_ENTRY(RomaImplDirAccess::~RomaImplDirAccess, << TRACE(this));

    delete[] (char*)m_data.Id;
    delete[] (char*)m_data.Password;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplDirAccess::RomaImplDirAccess(const RomaImplDirAccess& other)
{
    TRACE_ENTRY(RomaImplDirAccess::RomaImplDirAccess, << TRACE(this));

    kxcSetDirAccessOptDefault(&m_data);
    *this = other;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplDirAccess& 
RomaImplDirAccess::operator = (const RomaImplDirAccess& other)
{
    TRACE_ENTRY(RomaImplDirAccess::operator =, << TRACE(this));

    if (this != &other)
    {
        setUserId(other.getUserId());
        setPassword(other.getPassword());
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Set an user ID.
//--------------------------------------------------------------------
bool RomaImplDirAccess::setUserId(const char* id)
{
    TRACE_ENTRY(RomaImplDirAccess::setUserId, << TRACE(this)
                                              << TRACE(id));

    delete[] (char*)m_data.Id;
    m_data.Id = NULL;

    if (id)
    {
        size_t sz = strlen(id) + 1;
        m_data.Id = (PKXCCHAR)new char[sz];

        if (!m_data.Id)
        {
            // Out of memory.
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(sz));
            TRACE_INT_RETURN(false);
        }

        memcpy(m_data.Id, id, sz);
    }

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Set a password.
//--------------------------------------------------------------------
bool RomaImplDirAccess::setPassword(const char* pwd)
{
    TRACE_ENTRY(RomaImplDirAccess::setPassword, << TRACE(this));

    delete[] (char*)m_data.Password;
    m_data.Password = NULL;

    if (pwd)
    {
        size_t sz = strlen(pwd) + 1;
        m_data.Password = (PKXCCHAR)new char[sz];

        if (!m_data.Password)
        {
            // Out of memory.
            TRACE_WARNING(KXCOM_MEMALLOC_FAILED(sz));
            TRACE_INT_RETURN(false);
        }

        memcpy(m_data.Password, pwd, sz);
    }

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplDirAccess::trace(RomaTraceStream& s) const
{
    s << TRACE((const char*)m_data.Id);
}


//====================================================================
// The RomaImplMsgId class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplMsgId::RomaImplMsgId(const void* data) : m_msgIdPtr(m_msgId)
{
    if (data == NULL)
    {
        memset(m_msgId, 0, Roma::ID_LENGTH);
    }
    else
    {
        memcpy(m_msgIdPtr, data, Roma::ID_LENGTH);
    }
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplMsgId::RomaImplMsgId(const RomaImplMsgId& msgId) 
    : m_msgIdPtr(m_msgId)
{
    memcpy(m_msgIdPtr, msgId.m_msgIdPtr, Roma::ID_LENGTH);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplMsgId& RomaImplMsgId::operator = (const RomaImplMsgId& other)
{
    if (this != &other)
    {
        memcpy(m_msgIdPtr, other.m_msgIdPtr, Roma::ID_LENGTH);
    }

    return *this;
}

//--------------------------------------------------------------------
// Set Message ID data.
//--------------------------------------------------------------------
void RomaImplMsgId::set(const void* data, int len)
{
    if (data)
    {
        // Copy the specified data. If data length is greater than
        // Roma::ID_LENGTH, cut it to fit to the buffer.
        memcpy(m_msgIdPtr, data, min(Roma::ID_LENGTH, len));
    }
    else
    {
        // The specified pointer is NULL, so we just fill the whole
        // buffer with zeros.
        len = 0;
    }

    // Fill an unused part of the buffer with zeros.
    if (len < Roma::ID_LENGTH)
    {
        memset(m_msgIdPtr + len , 0, Roma::ID_LENGTH - len);
    }
}

//--------------------------------------------------------------------
// Copy Message ID data to the specified buffer.
//--------------------------------------------------------------------
void RomaImplMsgId::copy(void* buffer, int length) const
{
    if (buffer)
    {
        memcpy(buffer, m_msgIdPtr, min(Roma::ID_LENGTH, length));
    }
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplMsgId::operator == (const RomaImplMsgId& msgId) const
{
    return (memcmp(m_msgIdPtr, msgId.m_msgIdPtr, Roma::ID_LENGTH) == 0);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplMsgId::operator != (const RomaImplMsgId& msgId) const
{
    return (memcmp(m_msgIdPtr, msgId.m_msgIdPtr, Roma::ID_LENGTH) != 0);
}

//--------------------------------------------------------------------
// Fill the whole Message ID data buffer with zeros.
//--------------------------------------------------------------------
void RomaImplMsgId::setNull()
{
    memset(m_msgIdPtr, 0, Roma::ID_LENGTH);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplMsgId::trace(RomaTraceStream& s) const
{
    s << m_msgIdPtr << '-';

    for (int i = 0; i < Roma::ID_LENGTH; ++i) 
    {
        char ch = m_msgIdPtr[i];
        s << (isprint(ch) ? ch : '.');
    }
}


//====================================================================
// The RomaImplILSData class.
//====================================================================

RomaImplILSData::RomaImplILSData()
    : m_dataSet(NULL_HKXCILS), m_context(NULL), m_bufOwner(NULL_HKXC)
{
    kxcSetILSDefault(&m_options);
    kxcSetILSType(&m_options, KXCXIT_TYPE_SERVICE);
}

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplILSData::RomaImplILSData(HKXCILS dataSet,
                                 RomaImplContext* cxt,
                                 HKXC owner)
    : m_dataSet(dataSet), m_context(cxt), m_bufOwner(owner)
{
    TRACE_ENTRY(RomaImplILSData::RomaImplILSData, << TRACE(this)
                                                  << TRACE(dataSet));

    if (m_context != NULL) m_context->lockInstance();

    kxcSetILSDefault(&m_options);
    kxcSetILSType(&m_options, KXCXIT_TYPE_SERVICE);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplILSData::~RomaImplILSData()
{
    TRACE_ENTRY(RomaImplILSData::~RomaImplILSData, << TRACE(this));

    if (m_context != NULL)
    {
        if (m_context->isConnected() &&
            m_bufOwner == (HKXC)(*m_context))
        {
            kxcFreeBuffer(NULL_HKXC, &m_options.DataBuf);
        }

        m_context->releaseInstance();
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Indicate whether data may be replaced.
//--------------------------------------------------------------------
void RomaImplILSData::setReplace(bool b)
{
    kxcSetILSOptions(&m_options, (b ? KXCIOPT_REPLACE : KXCIOPT_NONE));
}

//--------------------------------------------------------------------
// Set ILS data buffer.
//--------------------------------------------------------------------
bool RomaImplILSData::setBuffer(void* buffer, 
                                int length, 
                                int method, 
                                Roma::ILSType type)
{
    TRACE_ENTRY(RomaImplILSData::setBuffer, << TRACE(this)
                                            << TRACE(buffer)
                                            << TRACE(length)
                                            << TRACE(method)
                                            << TRACE(type));

    // Setup ILS data options.
    kxcSetILSBuffer(&m_options, length, buffer);
    kxcSetILSLength(&m_options, length);
    kxcSetILSMethod(&m_options, method);
    kxcSetILSType(&m_options, type);

    // Set ILS data.
    bool success = (kxcSetILSData(&m_options, &m_dataSet) == TRUE);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get the first ILS data buffer.
//--------------------------------------------------------------------
const void* RomaImplILSData::getBuffer(int& length, 
                                       int method, 
                                       Roma::ILSType type)
{
    TRACE_ENTRY(RomaImplILSData::getBuffer, << TRACE(this)
                                            << TRACE(method)
                                            << TRACE(type));

    // Setup ILS data options.
    kxcSetILSType(&m_options, type);
    kxcSetILSMethod(&m_options, method);

    const void* data = NULL;

    // Get ILS data.
    if (kxcGetILSData(&m_options, m_dataSet))
    {
        // Extract data pointer and length.
        length = kxcGetILSLength(&m_options);
        data = kxcGetILSBuffer(&m_options);
    }
    else
    {
        // No data.
        length = 0;
        TRACE_WARNING(KXCOM_ILSDATA_FAILED);
    }

    TRACE_INT_RETURN(data);
}

//--------------------------------------------------------------------
// Get the next ILS data buffer.
//--------------------------------------------------------------------
const void* RomaImplILSData::getNextBuffer(int& length, 
                                           int& method, 
                                           Roma::ILSType& type)
{
    TRACE_ENTRY(RomaImplILSData::getNextBuffer, << TRACE(this));

    // Setup ILS data options.
    kxcSetILSDefault(&m_options);
    kxcSetILSOptions(&m_options, KXCIOPT_GETNEXT);

    const void* data = NULL;

    // Get ILS data.
    if (kxcGetILSData(&m_options, m_dataSet))
    {
        // Extract ILS data details.
        length = kxcGetILSLength(&m_options);
        method = kxcGetILSMethod(&m_options);
        type = (Roma::ILSType)kxcGetILSType(&m_options);
        data = kxcGetILSBuffer(&m_options);
    }
    else
    {
        // No data.
        length = 0;
        TRACE_WARNING(KXCOM_ILSDATA_FAILED);
    }

    TRACE_INT_RETURN(data);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplILSData::trace(RomaTraceStream& s) const
{
    void* buffer = m_options.DataBuf.pBuffer;
    s << TRACE(buffer);
}


//====================================================================
// The RomaImplLogInfo class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplLogInfo::RomaImplLogInfo()
{
    TRACE_ENTRY(RomaImplLogInfo::RomaImplLogInfo, << TRACE(this));

    KXCLOGDATA tmp = { KXCLOGDATA_DEFAULT };
    m_data = tmp;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplLogInfo::~RomaImplLogInfo()
{
    TRACE_ENTRY(RomaImplLogInfo::~RomaImplLogInfo, << TRACE(this));

    delete[] (char*)m_data.pUserText;
    delete[] (char*)m_data.DataBuf.pBuffer;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set user text.
//--------------------------------------------------------------------
bool RomaImplLogInfo::setUserText(const char* text)
{
    TRACE_ENTRY(RomaImplLogInfo::setUserText, << TRACE(this) 
                                              << TRACE(text));

    delete[] (char*)m_data.pUserText;
    m_data.pUserText = NULL;

    if (!text)
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_INT_RETURN(true);
    }

    size_t length = strlen(text) + 1;

    delete[] (char*)m_data.pUserText;
    m_data.pUserText = (PKXCCHAR)new char[length];
    bool success = (m_data.pUserText != NULL);

    if (success) 
    {
        memcpy(m_data.pUserText, text, length);
    }
    else
    {
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(length));
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Set user data.
//--------------------------------------------------------------------
bool RomaImplLogInfo::setUserData(const void* data, int length)
{
    TRACE_ENTRY(RomaImplLogInfo::setUserData, << TRACE(this) 
                                              << TRACE(data)
                                              << TRACE(length));

    if (!data || length <= 0)
    {
        TRACE_INT_RETURN(false);
    }

    delete[] (char*)m_data.DataBuf.pBuffer;
    m_data.DataBuf.pBuffer = new char[length];

    bool success = (m_data.DataBuf.pBuffer != NULL);

    if (success) 
    {
        memcpy(m_data.DataBuf.pBuffer, data, length);
        m_data.DataBuf.BufLen = m_data.DataLen = length;
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(length));
        m_data.DataBuf.BufLen = m_data.DataLen = 0;
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplLogInfo::trace(RomaTraceStream& s) const
{
    s << TRACE(m_data.Format)
      << TRACE(m_data.Severity)
      << TRACE(m_data.pUserText)
      << TRACE(m_data.DataLen)
      << TRACE(m_data.DataBuf.BufLen)
      << TRACE(m_data.DataBuf.pBuffer);
}


//====================================================================
// The RomaImplSyncpoint class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplSyncpoint::RomaImplSyncpoint(Roma::SyncType type)
    : m_type(type), m_context(NULL)
{
}

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplSyncpoint::RomaImplSyncpoint(PKXCVOID context) 
    : m_type(Roma::SYNC_GLOBAL), m_context(context)
{
}

//--------------------------------------------------------------------
// Fill the KXCSYNC structure with object data.
//--------------------------------------------------------------------
void RomaImplSyncpoint::populate(KXCSYNC& def) const
{
    def.TransactionType = m_type;
    def.TransactionContext = m_context;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplSyncpoint::trace(RomaTraceStream& s) const
{
    s << TRACE(m_type) << TRACE(m_context);
}


//====================================================================
// The RomaImplTheme class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplTheme::RomaImplTheme(HKXCTHEME handle) : m_handle(handle)
{
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplTheme& RomaImplTheme::operator = (HKXCTHEME h)
{
    m_handle = h;
    return *this;
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTheme::operator == (const RomaImplTheme& other) const
{
    return (m_handle == other.m_handle);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTheme::operator != (const RomaImplTheme& other) const
{
    return (m_handle != other.m_handle);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplTheme::trace(RomaTraceStream& s) const
{
    s << m_handle;
}


//====================================================================
// The RomaImplTimestamp class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplTimestamp::RomaImplTimestamp(KXCULONG sec, KXCULONG msec)
    : m_sec(sec), m_msec(msec)
{
}

RomaImplTimestamp::RomaImplTimestamp(const KXCTIMESTAMP& ts)
    : m_sec(ts.Seconds), m_msec(ts.Millisecs)
{
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplTimestamp& 
RomaImplTimestamp::operator = (const KXCTIMESTAMP& ts)
{
    m_sec = ts.Seconds;
    m_msec = ts.Millisecs;
    return *this;
}

//--------------------------------------------------------------------
// Fill the KXCTIMESTAMP structure with object data.
//--------------------------------------------------------------------
void RomaImplTimestamp::populate(KXCTIMESTAMP& ts) const
{
    ts.Seconds = m_sec;
    ts.Millisecs = m_msec;
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator == (
                            const RomaImplTimestamp& other) const
{
    return (m_sec == other.m_sec && m_msec == other.m_msec);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator != (
                            const RomaImplTimestamp& other) const
{
    return (m_sec != other.m_sec || m_msec != other.m_msec);
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator < (
                            const RomaImplTimestamp& other) const
{
    return (m_sec < other.m_sec ||
            (m_sec == other.m_sec && m_msec < other.m_msec));
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator <= (
                            const RomaImplTimestamp& other) const
{
    return (m_sec <= other.m_sec ||
            (m_sec == other.m_sec && m_msec <= other.m_msec));
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator > (
                            const RomaImplTimestamp& other) const
{
    return (m_sec > other.m_sec ||
            (m_sec == other.m_sec && m_msec > other.m_msec));
}

//--------------------------------------------------------------------
// Comparison operator.
//--------------------------------------------------------------------
bool RomaImplTimestamp::operator >= (
                            const RomaImplTimestamp& other) const
{
    return (m_sec >= other.m_sec ||
            (m_sec == other.m_sec && m_msec >= other.m_msec));
}


//====================================================================
// The RomaImplSubject class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplSubject::RomaImplSubject() : m_style(Roma::STYLE_ROMA)
{
    *m_name = '\0';
}

//--------------------------------------------------------------------
// Set a subject name.
//--------------------------------------------------------------------
void RomaImplSubject::setName(const char* name)
{
    if (name && *name)
    {
        strncpy(m_name, name, sizeof(m_name) - 1);
        m_name[sizeof(m_name) - 1] = '\0';
    }
    else
    {
        *m_name = '\0';
    }
}

//--------------------------------------------------------------------
// Fill the KXCSUBJECT structure with object data.
//--------------------------------------------------------------------
void RomaImplSubject::populate(KXCSUBJECT& s) const
{
    s.Style = m_style;
    s.pSubject = (PKXCCHAR)m_name;
    s.DataLen = strlen(m_name);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplSubject::trace(RomaTraceStream& s) const
{
    s << TRACE(m_style) << TRACE(m_name);
}


//====================================================================
// The RomaImplSecItem class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
//--------------------------------------------------------------------
RomaImplSecItem::RomaImplSecItem(int n, const void* p, int l)
    : m_method(n), m_data(NULL)
{
    TRACE_ENTRY(RomaImplSecItem::RomaImplSecItem, << TRACE(this) 
                                                  << TRACE(n));
    replaceData(p, l);
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplSecItem::~RomaImplSecItem()
{
    TRACE_ENTRY(RomaImplSecItem::~RomaImplSecItem, << TRACE(this));

    delete[] (char*)m_data;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set/replace item data.
//--------------------------------------------------------------------
bool RomaImplSecItem::replaceData(const void* p, int l)
{
    TRACE_ENTRY(RomaImplSecItem::replaceData, << TRACE(this) 
                                              << TRACE(p)
                                              << TRACE(l));

    delete[] (char*)m_data;
    m_length = 0;

    if (!p || !l)
    {
        m_data = NULL;
        TRACE_INT_RETURN(true);
    }

    m_data = new char[l];
    bool success = (m_data != NULL);

    if (success)
    {
        m_length = l;
        memcpy(m_data, p, l);
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(l));
    }

    TRACE_INT_RETURN(success);
}


//====================================================================
// The RomaImplSecurity class.
// RomaSecurityDef objects are always created by the application. When
// one of these objects is handed off to another object (e.g. Client
// Request) it is cloned, at which time the KXCSECDEF structure is
// built from the items defined thus far.
//====================================================================

//--------------------------------------------------------------------
// Define type aliases to make code more readable.
//--------------------------------------------------------------------
typedef RomaImplSecItem Item;
typedef RomaImplSecItemLList List;
typedef RomaImplSecItemLListIterator Iterator;

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplSecurity::RomaImplSecurity() 
    : m_pSecDef(NULL), m_total(0), m_cloned(false)
{
    TRACE_ENTRY(RomaImplSecurity::RomaImplSecurity, << TRACE(this));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplSecurity::~RomaImplSecurity()
{
    TRACE_ENTRY(RomaImplSecurity::~RomaImplSecurity, << TRACE(this));

    clear();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Delete object data.
//--------------------------------------------------------------------
void RomaImplSecurity::clear()
{
    TRACE_ENTRY(RomaImplSecurity::clear, << TRACE(this));

    delete[] (char*)m_pSecDef;
    m_pSecDef = NULL;
    m_total = 0;
    m_cloned = false;

    Iterator it(m_list);
    Item* item;

    while (it.GetNext(&item)) 
    {
        delete item;
    }

    m_list.Clear();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Create the C API representation of the security definition data.
//--------------------------------------------------------------------
bool RomaImplSecurity::consolidate(RomaImplSecurity& copy) const
{
    TRACE_ENTRY(RomaImplSecurity::consolidate, << TRACE(this) 
                                               << TRACE(copy));

    // For each ILS, a separate data buffer can be defined. The 
    // buffers are combined into a single array-like structure
    // (KXCSECDEF) which is then ready to be passed to the C API.
    Iterator it(m_list);
    Item* item;

    // Setup initial length - the standard KXCSECDEF structure.
    size_t length = sizeof(KXCSECDEF);

    // Add a KXCSECHDR and relevant data length for each buffer 
    // defined.
    unsigned count = 0;

    for (; it.GetNext(&item); ++count)
    {
        length += (sizeof(KXCSECHDR) + 
            kxcRoundLength(item->m_length));
    }

    // Delete existing data.
    delete[] (char*)copy.m_pSecDef;
    copy.m_total = 0;

    if (!count)
    {
        // No items, noting to do.
        copy.m_pSecDef = NULL;
        TRACE_EVENT(KXCOM_NO_ITEMS);
        TRACE_INT_RETURN(true);
    }

    // Create new security structure.
    copy.m_pSecDef = (KXCSECDEF*)new char[length];

    bool success = (copy.m_pSecDef != NULL);

    if (success) 
    {
        // Save size of the structure.
        copy.m_total = length;

        // Fill memory with zeros.
        memset(copy.m_pSecDef, 0, length);

        // Initialize the KXCSECDEF structure.
        // The length will be stored in m_pSecDef->TotalLen.
        kxcInitSecDef(copy.m_pSecDef, count, length);

        // Initialize individual security structures. These are 
        // effectively array entries (indexed from 0) and appended 
        // to the KXCSECHDR.
        it.Reset();
        for (count = 0; it.GetNext(&item); ++count) 
        {
            kxcInitSecHdr(copy.m_pSecDef, count, item->m_method,
                item->m_length, item->m_data);
        }
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(length));
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Applications always create the real RomaSecurityDef objects, and 
// are not allowed to clone them or access the KXCSECDEF structure.
// Other object model components deal with cloned RomaSecurityDef
// objects, and cannot modify the ILS data.
// The cloning mechanism fulfills several design requirements:
// 1. all knowledge about KXCSECDEF remains in RomaSecurityDef
// 2. RomaSecurityDef can be built incrementally, but a read-only
//    snap-shot is taken at the point that it is cloned
// 3. all memory management is handled by RomaSecurityDef
//--------------------------------------------------------------------
bool RomaImplSecurity::populate(RomaSecurityDef& copy) const
{
    TRACE_ENTRY(RomaImplSecurity::populate, << TRACE(this));

    RomaImplSecurity& impl = RomaImplProxy::getImpl(copy);
    impl.m_cloned = true;

    if (!m_cloned)
    {
        bool s = consolidate(impl);
        TRACE_INT_RETURN(s);
    }

    // Clear existing object data.
    impl.clear();

    // Allocate memory for security definition data.
    impl.m_pSecDef = (KXCSECDEF*)new char[m_total];
    bool success = (impl.m_pSecDef != NULL);

    if (success)
    {
        // Copy security definition data to the target object.
        memcpy(impl.m_pSecDef, m_pSecDef, m_total);
        impl.m_total = m_total;
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(m_total));
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Define a new buffer for the specified ILS.  This can only be 
// called on an 'original' (non-cloned) RomaSecurityDef object.
// A copy of the data is taken. Any previous data defined for the 
// same ILS is replaced.
//--------------------------------------------------------------------
bool RomaImplSecurity::setBuffer(int method, 
                                 const void* data, 
                                 int length)
{
    TRACE_ENTRY(RomaImplSecurity::setBuffer, << TRACE(this)
                                             << TRACE(method)
                                             << TRACE(data)
                                             << TRACE(length));

    if (m_cloned)
    {
        // This method should not be called for a cloned
        // security definition.
        TRACE_INT_RETURN(false);
    }

    RomaImplSecItem* item;
    Iterator it(m_list);

    // Walk through the items to find one associated with the
    // specified ILS identifier (method).
    while (it.GetNext(&item))
    {
        if (item->m_method == method) 
        {
            // The target item found.
            bool res = item->replaceData(data, length);
            TRACE_INT_RETURN(res);
        }
    }

    // The target item is not found, so create new one and add to
    // the list.
    RomaImplSecItem* newItem = 
        new RomaImplSecItem(method, data, length);
    bool success = (newItem != NULL);

    if (success)
    {
        m_list.Append(newItem);
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_NEW_FAILED);
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Return the pointer the KXCSECDEF structure containing object data
// in the C API representation.
//--------------------------------------------------------------------
RomaImplSecurity::operator PKXCSECDEF() const
{
    TRACE_ENTRY(RomaImplSecurity::operator PKXCSECDEF, 
        << TRACE(this));

    if (!m_cloned)
    {
        // The object is not cloned, so we need to create the C API
        // representation of its data.
        consolidate((RomaImplSecurity&)(*this));
    }

    TRACE_RETURN((PKXCSECDEF)m_pSecDef);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplSecurity::trace(RomaTraceStream& s) const
{
    s << TRACE(m_pSecDef) << TRACE(m_total) << TRACE(m_cloned)
      << "items: ";

    Iterator it(m_list);
    Item* item;

    while (it.GetNext(&item)) 
    {
        s << "[" << item->m_method << "," << item->m_data << ","
          << item->m_length << "] ";
    }
}

