//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaDirectoryAgent.java
//
// Defines      : class RomaDirectoryAgent
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class encapsalates Roma directory access. 
* Each instance uses the supplied context to access
* the Roma directory and is constructed with
* the required search criteria. More than one instance 
* may be created using the same context.
* <p>
* A directory item is retrieved by calling the item() method 
* which constructs a RomaDirectoryItem, methods on this are used
* to access data within the item.
* @see RomaDirectoryItem 
*/

public class RomaDirectoryAgent
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2002";

  /**
  * Constructor for client with no search options
  *     
  * @param client	     should specify a valid client
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType	 search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * 
  * @exception  RomaError  if the search fails.
  */
  public RomaDirectoryAgent( RomaClient client,
                             String			startingPoint,
                             String			searchName,
                             int            searchType )
    throws RomaError
  {
    m_ctx           = client;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = Roma.USE_NO_BASE;
    m_matches       = 0;
  }

  /**
  * Constructor for client with search options
  *     
  * @param context       should specify a valid Client or Business resp
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * @param options       search option. One of the following flags
  * <ul>
  *   <li>OPTION_NONE    uses startingPoint as start of search
  *   <li>OPTION_BASE    searches at top of tree, ignoring startingPoint
  * 
  * @exception  RomaError  if the search fails.
  */
  public RomaDirectoryAgent( RomaClient client,
                             String     startingPoint,
                             String     searchName,	
                             int        searchType,
                             int        options)
    throws RomaError
  {
    m_ctx           = client;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = options;
    m_matches       = 0;
  }
  
  /**
  * Constructor for business element with no search options
  *     
  * @param client        should specify a valid client
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * 
  * @exception  RomaError  if the search fails.
  */
  public RomaDirectoryAgent( RomaBusinessElement element,
                             String              startingPoint,
                             String              searchName,	
                             int                 searchType )
    throws RomaError
  {
    m_ctx           = element;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = Roma.USE_NO_BASE;
    m_matches       = 0;
  }
  
  /**
  * Constructor for business element with search options
  *     
  * @param context       should specify a valid Client or Business resp
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * @param options       search option. One of the following flags
  * <ul>
  *   <li>OPTION_NONE    uses startingPoint as start of search
  *   <li>OPTION_BASE    searches at top of tree, ignoring startingPoint
  * 
  * @exception  RomaError  if the search fails.
  */
  public RomaDirectoryAgent( RomaBusinessElement element,
                             String              startingPoint,
                             String              searchName,	
                             int                 searchType,
                             int                 options)
    throws RomaError
  {
    m_ctx           = element;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = options;
    m_matches       = 0;
  }
  
  /**
  * set new parameters for directory search
  *     
  * @param client        should specify a valid client
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * 
  * @exception  RomaError  if the search fails.
  */
  public void lookup( RomaClient client,
                      String     startingPoint,
                      String     searchName,	
                      int        searchType )
    throws RomaError
  {
    m_ctx           = client;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = Roma.USE_NO_BASE;
    m_matches       = 0;
  }
  
  /**
  * set new parameters for directory search
  *     
  * @param context       should specify a valid Client or Business resp
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * @param options       search option. One of the following flags
  * <ul>
  *   <li>OPTION_NONE    uses startingPoint as start of search
  *   <li>OPTION_BASE    searches at top of tree, ignoring startingPoint
  * 
  * @exception  RomaError  if the search fails.
  */
  public void lookup( RomaClient client,
                      String     startingPoint,
                      String     searchName,	
                      int        searchType,
                      int        options)
    throws RomaError
  {
    m_ctx           = client;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = options;
    m_matches       = 0;
  }

  /**
  * set new parameters for directory search
  *     
  * @param client        should specify a valid client
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * 
  * @exception  RomaError  if the search fails.
  */
  public void lookup( RomaBusinessElement element,
                      String              startingPoint,
                      String              searchName,	
                      int                 searchType )
    throws RomaError
  {
    m_ctx           = element;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = Roma.USE_NO_BASE;
    m_matches       = 0;
  }
  
  /**
  * set new parameters for directory search
  *
  * @param context       should specify a valid Client or Business resp
  * @param startingPoint path to starting point of search
  * @param searchName    search for this name
  * @param searcType     search type. One of the following flags
  * <ul>
  *   <li>SEARCH_EXACTMATCH
  *   <li>SEARCH_ONELEVEL
  *   <li>SEARCH_SUBTREE
  * </ul>
  * @param options       search option. One of the following flags
  * <ul>
  *   <li>OPTION_NONE    uses startingPoint as start of search
  *   <li>OPTION_BASE    searches at top of tree, ignoring startingPoint
  * 
  * @exception  RomaError  if the search fails.
  */
  public void lookup( RomaBusinessElement element,
                      String              startingPoint,
                      String              searchName,	
                      int                 searchType,
                      int                 options)
    throws RomaError
  {
    m_ctx           = element;
    m_startingPoint = startingPoint;
    m_searchName    = searchName;
    m_searchType    = searchType;
    m_options       = options;
    m_matches       = 0;
  }
  
  /**
  * Returns number of matching items and modified flag.
  * 
  * @param modified non-zero value means directory was modified
  * @return number of matching item
  * @exception  RomaError is thrown on operation failure.
  */
  public int matches(RomaInteger modified )
    throws RomaError
  {		  
    try {  
      RomaContext.kxcOper( m_ctx.kxcCtx().dirLookUp(m_startingPoint, 
                                                    m_searchName,  
                                                    m_searchType, 
                                                    m_options));
    }
    catch (RomaError xRoma) {
      // Want to return 0 matches when no matching entries in directory.
      // If the search name cannot be found a primary return code 
      // RC_UNKNOWN_OBJECT (77) is returned. If the StartingPoint does 
      // not exist then the secondary error is also set with
      // return code of RC_LDIF_GET_FAILURE (120).
      // Return 0 matches for both cases.
      if ( xRoma.returnCode( Kxc.PRIMARY_INFO ) == Roma.RC_UNKNOWN_OBJECT )	
      {
        m_matches = 0;	
        return 0;
      }
      throw xRoma;					
    }

    // retrieve matching items count and modification count
    RomaInteger nModCount = new RomaInteger();
    m_matches = m_ctx.kxcCtx().dirLookupReturn( nModCount );
    
    modified.setValue( (m_modCount == nModCount.intValue() ) ? 0 : 1);
    
    m_modCount = nModCount.intValue();
    return m_matches;
  }
  
  /**
  * Returns number of matching items.
  *   
  * @return number of matching item
  * @exception  RomaError is thrown on operation failure.
  */
  public int matches()
    throws RomaError
  {
    RomaInteger modified = new RomaInteger(0);		
    return matches( modified );
  }	
  
  /**
  * Retrieve nth item form the search.
  * 
  * @param nItem item required.
  * @return a RomaDirectoryItem
  * @exception  RomaError is thrown on operation failure.
  */
  public RomaDirectoryItem item( int nItem )
    throws RomaError
  {
      matches( );
	 	 
    RomaContext.kxcOper(m_ctx.kxcCtx().dirGetItem( nItem ));
    
    RomaInteger length = new RomaInteger();
    byte[] buffer = m_ctx.kxcCtx().dirGetItemReturn( length );
    
    RomaDirectoryItem romaDirItem = new RomaDirectoryItem( buffer, 
                                                           length.intValue() );
    
    return romaDirItem; 
  }
  
  /**
  * Retrieve nth item form the search and flag whether
  * the directory has been modified.
  * 
  * @param nItem      item required.
  * @param mModified  will be set to 1 if directory modified, else 0
  * @return           a RomaDirectoryItem
  * @exception  RomaError is thrown on operation failure.
  */
  public RomaDirectoryItem item(int         nItem, 
                                RomaInteger nModified )
    throws RomaError
  {
    RomaDirectoryItem romaDirItem = item( nItem );
    
    // set nModified non-zero if modification count has changed
    // since search was done.
    RomaContext.kxcOper(m_ctx.kxcCtx().dirModCount( ));
    int newModCount = m_ctx.kxcCtx().dirModCountReturn( );
    
    int n;
    if ( newModCount != m_modCount )
      n = 1;
    else 
      n = 0;
    nModified.setValue( n );
    
    return romaDirItem; 
  }
  
  /**
  * Initialise as a new item with given distiguished name.
  * Package visibility.
  * 
  * @param      distName distiguished name   
  * @exception  RomaError is thrown on operation failure.
  */
  RomaDirectoryItem newItem( String distName )
    throws RomaError
  {
    RomaDirectoryItem item = new RomaDirectoryItem();
    item.setDN(distName);
    return item;
  }

  /**
  * Store item with default options. Directory access credential
  * should be specified for RomaClient::connect() or
  * RomaBusinessElement::connect().
  *
  * @param romaDI    item to store.
  *
  * @exception  RomaError is thrown on operation failure.
  */
  public void storeItem( RomaDirectoryItem  romaDI )
    throws RomaError
  {
    romaDI.storeItem( m_ctx, null, null );
  }

  /**
  * Store item with default options
  *
  * @param romaDI    item to store.
  * @param id        security identity
  * @param password  security password
  *
  * @exception  RomaError is thrown on operation failure.
  */
  public void storeItem( RomaDirectoryItem  romaDI,
                         String             id,
                         String             password )
    throws RomaError
  {
    romaDI.storeItem( m_ctx, id, password );
  }


  /**
  * Delete the item with given distiguished name.
  * Directory access credential should be specified for
  * RomaClient::connect() or RomaBusinessElement::connect().
  *
  * @param      distName distiguished name
  * @exception  RomaError is thrown on operation failure.
  */
  public void deleteItem( String distName )
    throws RomaError
  {
     deleteItem( distName, null, null );
  }


  /**
  * Delete the item with given distiguished name.
  *
  * @param      distName distiguished name
  * @param id        security identity
  * @param password  security password
  *
  * @exception  RomaError is thrown on operation failure.
  */
  public void deleteItem( String distName,
                          String id,
                          String password )
    throws RomaError
  {
    // Build an item buffer with distinguished name and
    // submit to JNI delete function
    int len = RomaDirectoryItem.BUF_HEADER_SIZE +
       RomaDirectoryItem.BUF_RES_SIZE + distName.length();
    byte[] buffer = new byte[len];

    buffer = KxcContext.dirSetDN(buffer, len, distName);

    RomaContext.kxcOper( m_ctx.kxcCtx().dirDeleteItem(
        buffer, len, Roma.FORCE, id, password ));
  }

  /**
  * Process each item in the current selection.
  *
  * The method called action will be invoked for
  * each item. The application can provide an
  * implementation in a derived class.
  * @return number of processed items
  * @exception  RomaError is thrown on operation failure.
  *
  * @see #action
  */
  public int processAll() throws RomaError
  {
    int n = matches();
    int i = 0;

    for (; i < n; i++)
    {
      if (!action(item(i)))
      {
        break;
      }
    }

    return i;
  }

  /**
  * Invoked for each item in the current selection
  * when processAll is called.
  *
  * The application can provide an implementation in a derived
  * class.
  * @return true is further items are to be processed.
  * 
  * @see #processAll
  */
  protected boolean action(RomaDirectoryItem item)
  {
    return true;
  }
  
  // context for Roma operations
  private RomaContext m_ctx;
  // search criteria ...
  private String m_startingPoint;		 
  private String m_searchName;
  private int    m_searchType;
  private int    m_options;
  
  // Number of matching items returned by search
  private int m_matches;
  
  // Modification count returned by search
  private int m_modCount;
}

//--------------------------------------------------------------------
// End of file.
//--------------------------------------------------------------------
