//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaReportOptions.java
//
// Defines      : class RomaReportOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class contains the options that the application supplies
* when it invokes the Error Analysis Routine.
*/
public class RomaReportOptions
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructs a ReportOptions object with default attributes.
  */
  public RomaReportOptions()
  {
  }
  
  /**
  * Sets the error profile name.
  * @param errorProfileName the new error profile name
  */
  public void setErrorProfileName( String errorProfileName )
  {
    m_errorProfileName = errorProfileName;
  }
  
  /**
  * Sets the default actions that should be carried out.
  * This overrides the actions listed in the profile.
  *
  * @param defaultActions  any combination of the 
  * following flags :-
  * <ul>
  *   <li>ACTION_NOTIFY - send notification response
  *   <li>ACTION_EMS - put to exception message service
  *   <li>ACTION_ALERT - send alert
  *   <li>ACTION_LOG - write to log
  *   <li>ACTION_RESET - reset error
  *   <li>ACTION_NOACTION - take no action
  *   <li>ACTION_EMSRAW - put raw message to exception message
  *       service.
  * </ul>
  */
  public void setDefaultActions( int defaultActions )
  {
    m_defaultActions = defaultActions;
  }
  
  /**
  * Enable or disable the option to put the message to the 
  * exception management service if the report error operation
  * fails.
  *
  * @param flag  boolean selector.
  * <ul>
  *   <li>true  - disable put to the exception message service.
  *   <li>false - enable put to the exception message service.
  * </ul>
  */
  public void setNoEms( boolean flag )
  {
    if (flag)
      m_opts |= Kxc.EPOPT_NOEMS;
    else
      m_opts &= ~Kxc.EPOPT_NOEMS;
  }
  
  /**
  * Enable or disable the option to send a notification message
  * to the client if the report error operation fails.
  *
  * @param flag  boolean selector.
  * <ul>
  *   <li>true  - disable the sending of a notification.
  *   <li>false - enable the sending of a notification.
  * </ul>
  */
  public void setNoNotification( boolean flag )
  {
    if (flag)
      m_opts |= Kxc.EPOPT_NONOTIFY;
    else
      m_opts &= ~Kxc.EPOPT_NONOTIFY;
  }
  
  /**
  * Enable or disable the option to send an error profiling message
  * to a transformation engine if the message type is not 
  * recognised.
  *
  * @param flag  boolean selector.
  * <ul>
  *   <li>true  - prevent involvement of the transformation engine.
  *   <li>false - allow use of a transformation engine.
  * </ul>
  *
  */
  public void setNoTransformation( boolean b )
  {
    if (b)
      m_opts |= Kxc.EPOPT_NOXFORM;
    else
      m_opts &= ~Kxc.EPOPT_NOXFORM;
  }
  
  /**
  * Enable or disable the option to prevent alerts from being
  * sent. 
  *
  * @param flag  boolean selector.
  * <ul>
  *   <li>true  - do not allow alerts to be sent.
  *   <li>false - allow alerts.
  * </ul>
  *
  */
  public void setNoAlert( boolean flag )
  {
    if (flag)
      m_opts |= Kxc.EPOPT_NOALERT;
    else
      m_opts &= ~Kxc.EPOPT_NOALERT;
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_sync = sync;
  }
  
  /**
  * Sets the theme.
  *
  * @param theme  the new theme.
  * 
  */
  public void setTheme( RomaTheme theme )
  {
    m_theme = theme;
  }
  
  /**
  * Return sync point definitiion
  */
  RomaSyncpointDef syncpointDef( )
  {
    return m_sync;
  }
  
  /**
  * @see RomaContext#reportError(RomaExceptionMessage,
  *        RomaReportOptions)
  */
  void prepare( KxcContext ctx )
  {
    KxcReportCtrl ctrl = ctx.reportCtrl();
    int opts           = m_opts;
    
    if (m_errorProfileName != null)
    {
      ctrl.setProfileName( m_errorProfileName );
      opts |= Kxc.EPOPT_USEEPNAME;
    }
    else if (m_defaultActions != Roma.ACTION_DEFAULT)
    {
      ctrl.setDefaultActions( m_defaultActions );
      opts |= Kxc.EPOPT_USEDEFAULT;
    }
    ctrl.setOptions( opts );
  }
  
  /**
  * @see RomaContext#reportError(RomaExceptionMessage,
  *        RomaReportOptions)
  */
  void process( KxcContext ctx )
  {
    m_requestedActions = ctx.reportCtrl().defaultActions();
  }
  
  RomaTheme theme() 
  { 
    return m_theme; 
  }
  
  private String  m_errorProfileName  = null;
  private int     m_defaultActions    = Roma.ACTION_DEFAULT;
  private int     m_requestedActions  = 0;
  private int     m_opts              = Kxc.EPOPT_NONE;
  
  private RomaSyncpointDef m_sync   = new RomaSyncpointDef( Roma.SYNC_PROFILE );
  private RomaTheme        m_theme  = null;
}
