//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaSecurityDef.java
//
// Defines      : class RomaSecurityDef
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import java.io.*;
import java.util.*;
import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the security information for an
* inline service.
*/
public class RomaSecurityDef
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Initializes an empty security definition.
  */
  public RomaSecurityDef()
  {
  }
  
  /**
  * Represents a single Roma security buffer within
  * the security definition, which consists of a ILS number 
  * and data.
  */
  public class SecBuffer 
  {
    private final String copyrightNotice =          
      "Copyright Candle Corporation 1999,2000,2001";
    
    /**
    * Constructs a new SecBuffer instance.
    * 
    * @param   method identifier.
    * @param pos  the position of the method in the security
    * 
    *   definition.
    */
    SecBuffer( int    nILSNumber, 
               byte[] data, 
               int    length )
    {
      m_nILSNumber = nILSNumber;
      m_data       = data;
      m_dataLength = length;
    }
    
    int     m_nILSNumber;   // Inline service number
    byte[]  m_data;         // data for this ILS
    int     m_dataLength;   // data length for this ILS
    
  } // end of inner class Method
  
  /**
  * Define a new buffer for the specified inline service	
  * 
  * @param ILSNumber inline service number
  * @param data the security data
  * @param length the length of security data 
  * 
  * @return     true if successful
  */
  public boolean setBuffer( int     ILSNumber, 
                            byte[]  data, 
                            int     length )
  {
    boolean bFoundExisting = false;
    
    for (int i =0; i < m_list.size(); i++)
    {
      SecBuffer sb = (SecBuffer)m_list.get( i );
      if ( sb.m_nILSNumber == ILSNumber )
      {
        // update existing entry 
        bFoundExisting  = true;	
        sb.m_data       = data;
        sb.m_dataLength = length;
        break;
      }
    }
    
    if ( !bFoundExisting )
    {
      // add a new entry to end of list	
      SecBuffer sb = new SecBuffer( ILSNumber, data, length);
      
      if (sb == null)
        return false;
      
      m_list.add( sb );
    }
    m_hasChanges = true;
    return true;
  }		
  
  /**
  * Returns the binary security definition buffer.
  * <p>Checks whether the definition has changed and, if so,
  * reconstructs the buffer.
  *
  * @exception RomaError  if security definition is invalid.
  */
  byte[] getBuffer()
    throws RomaError
  {
    if ( m_hasChanges )
    {
      m_hasChanges = false;
      int nItems = m_list.size();
      if ( nItems > 0 )
      {
        KxcOper oper = KxcContext.setSecurityBufferOper( m_list.size() );
        
        for ( int i = 0; i < nItems; i++ )
        {
          SecBuffer sb = (SecBuffer)m_list.get( i );
          
          KxcContext.setSecurityBuffer( oper, 
                                        nItems, 
                                        i + 1, 
                                        sb.m_nILSNumber,
                                        sb.m_data, 
                                        sb.m_dataLength );
        }
        
        try {  
          oper.invoke();
          m_buf = KxcContext.getSecurityBuffer( oper );		
        }
        catch ( Exception x ) {
        }
        
      }
      else
      {
        m_buf = null;
      }
    }
    return m_buf;
  }
  
  private ArrayList m_list     = new ArrayList();
  private boolean m_hasChanges = false;
  private byte    m_buf[]      = null;
}
