//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocrq.hpp,v 1.12 2003/02/21 22:48:03 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocrq.hpp,v $
// $Revision: 1.12 $
//--------------------------------------------------------------------
// The RomaClientRequest public interface.
//--------------------------------------------------------------------

#ifndef __KXCOCRQ_HPP
#define __KXCOCRQ_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcosec.hpp>
#include <kxcosyn.hpp>
#include <kxcothm.hpp>
#include <kxcomid.hpp>
#include <kxcoils.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplClientRequest;


//--------------------------------------------------------------------
// The RomaClientRequest class.
//
// This class encapsulates requests made by a Roma client to a Roma
// business service. It allows the application to set the options and
// the service required when making a client request.
// This class also provides a framework for associating application
// specific data with a client request. The RomaClientRequest class
// can be sub-classed to allow application specific data marshalling
// using the marshalRequest() method.
// Objects of this and derived classes are used by the
// RomaClient::put() method.
//--------------------------------------------------------------------
class KXCODLL RomaClientRequest : public RomaTypeBase, 
                                  public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructs the request with default options, the specified
    // buffers and service name.
    //
    // Parameters:
    //   (in) serviceName
    // [
    //   (in) data - pointer to supplied buffer (MsgBuffer)
    //   (in) dataLength - length of (relevant) data in the buffer
    //   (in) pointer to supplied buffer (TagBuffer)
    //   (in) length of data within the buffer
    // ]
    //----------------------------------------------------------------
    RomaClientRequest(const char* serviceName);

    RomaClientRequest(const char* serviceName,
                      const void* msgBuffer,
                      int         msgBufferLength);

    RomaClientRequest(const char* serviceName,
                      const void* msgBuffer,
                      int         msgBufferLength,
                      const void* tagBuffer,
                      int         tagBufferLength);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaClientRequest();

    //----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create an independent copy of the
    // message, but just share the common implementation instance.
    //----------------------------------------------------------------
    RomaClientRequest(const RomaClientRequest&);
    RomaClientRequest& operator = (const RomaClientRequest&);

    //----------------------------------------------------------------
    // Set a business service name.
    //
    // The service name must be defined before the put() method can be
    // called.
    //----------------------------------------------------------------
    void setServiceName(const char* name);

    //----------------------------------------------------------------
    // Set theme options.
    //
    // If a new theme is requested, put() uses a new theme for each
    // request. Otherwise the specified theme will be used.
    //
    // If close theme is requested, the theme is closed after the
    // request is sent. If retain theme is requested, the theme is
    // retained for future use.
    //
    // These selections may be overidden if an API profile is defined
    //----------------------------------------------------------------
    void setNewTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setRetainTheme(bool f = true);
    void setCloseTheme(bool f = true);

    //----------------------------------------------------------------
    // Controls the type of message:
    //   true  - DATAGRAM
    //   false - REQUEST (default)
    //
    // However, if an API profile is used (isApiProfileUsed() == true)
    // then either the default or the profile value is used.
    //----------------------------------------------------------------
    void setDatagram(bool flag = true);

    //----------------------------------------------------------------
    // Set or clear the following flags:
    //    Confirmation of delivery (COD)
    //    Confirmation of arrival  (COA)
    //----------------------------------------------------------------
    void setCOD(bool flag = true);
    void setCOA(bool flag = true);

    //----------------------------------------------------------------
    // Set the security definition to be used with this request.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef& secDef);

    //----------------------------------------------------------------
    // Get/set the Roma Theme ID.
    //----------------------------------------------------------------
    const RomaTheme& theme() const;
    void  setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Get/set the Roma Message ID.
    //----------------------------------------------------------------
    const RomaMsgId& msgId() const;
    void  setMsgId(const RomaMsgId& msgId);

    //----------------------------------------------------------------
    // Get/set the Roma Correlation ID.
    //----------------------------------------------------------------
    const RomaMsgId& correlId() const;
    void  setCorrelId(const RomaMsgId& correlId);

    //----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //----------------------------------------------------------------
    void setPriority(int priority);

    //----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //----------------------------------------------------------------
    // Set the format number.
    //  If FORMAT_PROFILE is specified then format is retrieved from
    //  the API profile, defaulting to FORMAT_BINARY.
    //----------------------------------------------------------------
    void setFormat(int format);

    //----------------------------------------------------------------
    // Set the feedback code.
    //  If FEEDBACK_PROFILE is specified then code is retrieved from
    //  the API profile, defaulting to FEEDBACK_NONE.
    //----------------------------------------------------------------
    void setFeedback(int feedback);

    //----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //----------------------------------------------------------------
    void setExpiry(int expiry);

    //----------------------------------------------------------------
    // Set the code page.
    //  If CODEPAGE_PROFILE is specified then codepage number is
    //  retrieved from the API profile, defaulting to CODEPAGE_USER
    //----------------------------------------------------------------
    void setCodepage(int codepage);

    //----------------------------------------------------------------
    // Set the encoding.
    //  If ENCODING_PROFILE is specified then encoding is
    //  retrieved from the API profile, defaulting to ENCODING_DEFAULT
    //----------------------------------------------------------------
    void setEncoding(int encoding);

    //----------------------------------------------------------------
    // Set the format name to the specified string and format name to
    // FORMAT_USENAME.
    //----------------------------------------------------------------
    void setFormatName(const char* name);

    //----------------------------------------------------------------
    // Set the message name.
    // Defines a message name passed on to the Application Management
    // inline service.
    //----------------------------------------------------------------
    void setMsgName(const char* name);

    //----------------------------------------------------------------
    // Set message and tag data buffers.
    //----------------------------------------------------------------
    bool setMsgBuffer(const void* buffer, int dataLength);
    bool setTagBuffer(const void* buffer, int dataLength);

    //----------------------------------------------------------------
    // Sets the put selector.
    //
    // The selector specifies which identifiers contained in the
    // RomaMessage (msg id, correl id) to use when the message
    // is sent.
    //
    // The selector is specified by one of the following constants:
    //   USE_NONE     - doesn't set any specific selector
    //   USE_MSGID    - use Message Id as the selector
    //   USE_CORRELID - use Correlation Id as the selector
    //   USE_MSGID_AND_CORRELID - use both
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector selector);

    //----------------------------------------------------------------
    // Allows/disallows putting messages under transaction to a
    // non-transactional queue.
    //----------------------------------------------------------------
    void setNoSyncAllowed(bool flag = true);

    //----------------------------------------------------------------
    // Set API profile name. Supply null name to suppress use.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Sets error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // If specified, Roma does not put the message to exception
    // message service on error.
    //----------------------------------------------------------------
    void setNoEms(bool flag = true);

    //----------------------------------------------------------------
    // If notification is allowed, Roma can send responses straight
    // back to the client, ignoring any further business components.
    // (API profile may override)
    // The following method allows the application to specify that
    // notification messages may NOT be sent back to the client.
    //----------------------------------------------------------------
    void setNoNotification(bool flag = true);

    //----------------------------------------------------------------
    // If true, Roma does not send the message to the
    // Transformation Business Service if the next component in the
    // workflow cannot understand the message format being sent.
    // If the recipient will not accept this message format, and this
    // option is selected, Roma treats the Put operation as an error.
    //----------------------------------------------------------------
    void setNoTransformation(bool flag = true);

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool flag = true);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool flag = true);

    //----------------------------------------------------------------
    // Set the ILS Data definition to be used with this request.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData& ILSDataSet);

    //----------------------------------------------------------------
    // Return any inline service data related to this request.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;

    //----------------------------------------------------------------
    // Set a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Marshal the request.
    //
    // This method is intended to be overriden in derived classes.
    // The RomaClient::put() method calls this method before sending
    // the request. If the method returns false, the put() operation
    // doesn't send the request and returns an error.
    //----------------------------------------------------------------
    virtual bool marshalRequest();

    //----------------------------------------------------------------
    // Default constructor. For internal use only.
    //----------------------------------------------------------------
    RomaClientRequest();

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplClientRequest* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOCRQ_HPP

