//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoerr.hpp,v 1.13 2003/12/11 18:30:05 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoerr.hpp,v $
// $Revision: 1.13 $
//--------------------------------------------------------------------
// The RomaError public interface.
//--------------------------------------------------------------------

#ifndef __KXCOERR_HPP
#define __KXCOERR_HPP

#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplError;


//--------------------------------------------------------------------
// The RomaError class.
//
// This class contains information about the error that occurred as a
// result of a Roma operation. The class stores primary and
// secondary information about the error.
//--------------------------------------------------------------------
class KXCODLL RomaError : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaError();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaError();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaError(const RomaError&);
    RomaError& operator = (const RomaError&);

    //----------------------------------------------------------------
    // Indicates whether error information is set.
    //----------------------------------------------------------------
    bool anyError() const;

    //----------------------------------------------------------------
    // Get status of the error information. It is one of:
    //
    //   STATUS_NONE    - no error information is set.
    //   STATUS_PRIMARY - primary error information is set.
    //   STATUS_BOTH    - primary and secondary error information is
    //                    set.
    //----------------------------------------------------------------
    Roma::ErrorStatus status() const;

    //----------------------------------------------------------------
    // Get the completion code. Possible values are:
    //   CC_OK      - success
    //   CC_WARNING - partial success
    //   CC_FAILED  - fail
    //----------------------------------------------------------------
    Roma::CompCode compCode(
        Roma::ErrorSubclass c = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Get the return code.
    // It indicates a cause of the error.
    //----------------------------------------------------------------
    int returnCode(
        Roma::ErrorSubclass which = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Get the reason code.
    // Returns -1 if error information is not set.
    //----------------------------------------------------------------
    int reason(
        Roma::ErrorSubclass which = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Get the operation type.
    // Returns type of failed operation or -1 if error information is
    // not set.. Examples are:
    //   OP_KXC_OM   - Roma Object Model error
    //   OP_KXC      - Roma error
    //   OP_OS       - OS error
    //   OP_LIB      - standard library error
    //   OP_MSMQ     - MSMQ error
    //   OP_MQSERIES - MQSeries error
    //   OP_LDAP     - LDAP error
    //   OP_LDIF     - LDIF error
    //   OP_RPC      - RPC error
    //----------------------------------------------------------------
    int operationType(
        Roma::ErrorSubclass which = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Returns position of failed operation or -1 if error information
    // is not set. Use with additionalInfo() to uniquely identify the
    // failing code.
    //----------------------------------------------------------------
    int position(
        Roma::ErrorSubclass which = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Returns error type or -1 if error information is not set.
    //
    // It can be one of the following constants:
    //   TYPE_CRITICAL - critical error
    //   TYPE_TRANSIENT - transient error
    //   TYPE_ERROR - regular error
    //   TYPE_API - bad parameter was passed to method
    //----------------------------------------------------------------
    Roma::ErrorType errorType(
        Roma::ErrorSubclass which = Roma::SUBCLASS_PRIMARY) const;

    //----------------------------------------------------------------
    // Returns error description in suitable form for display, or NULL
    // if error information is not set.
    // format is one of:
    //     ERROR_FULL    - full error information
    //     ERROR_COMPACT - short error information
    //
    // NOTE: applications must not attempt to free the returned pointer.
    //       The returned string is valid until another call to
    //       errorText() is made on the same thread.
    //----------------------------------------------------------------
    const char* errorText(Roma::ErrorSubclass c = Roma::SUBCLASS_PRIMARY,
                          Roma::ErrorFormat f = Roma::ERROR_FULL) const;

    //----------------------------------------------------------------
    // Returns additional information / error location or NULL if
    // error information is not set.
    // Used in conjunction with position to uniquely identify the
    // failing code.
    //----------------------------------------------------------------
    const char* additionalInfo(
        Roma::ErrorSubclass c = Roma::SUBCLASS_PRIMARY) const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplError* m_impl;

private:
    RomaError(RomaImplError*);

    friend class RomaImplProxy;
};


#endif // __KXCOERR_HPP

