//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcorda.hpp,v 1.14 2004/01/22 17:53:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcorda.hpp,v $
// $Revision: 1.14 $
//--------------------------------------------------------------------
// The RomaDirectoryItem and the RomaDirectoryAgent public interfaces.
//--------------------------------------------------------------------

#ifndef __KXCORDA_HPP
#define __KXCORDA_HPP

#include <kxcocl.hpp>
#include <kxcobe.hpp>

//--------------------------------------------------------------------
// The internal implementation classes.
//--------------------------------------------------------------------
class RomaImplDirItem;
class RomaImplDirAgent;


//--------------------------------------------------------------------
// The RomaDirectoryItem class.
//
// Objects of type RomaDirectoryItem are used on two ways:
// 1. they are returned to an application when it queries the 
//    directory. The class supports methods to retrieve the data.
// 2. they are created by an application in order to update the
//    directory. The class supports methods to add new attributes.
//--------------------------------------------------------------------
class KXCODLL RomaDirectoryItem : public RomaTraceable,
                                  public RomaAllocator
{
public:
    //----------------------------------------------------------------
    // Construct the item with the specified distinguished name.
    //----------------------------------------------------------------
    RomaDirectoryItem(const char* dn, const char* objectClass);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaDirectoryItem();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaDirectoryItem(const RomaDirectoryItem&);
    RomaDirectoryItem& operator=(const RomaDirectoryItem&);

    //----------------------------------------------------------------
    // Set the distinguished name of this Item
    // (for adding a new item).
    //----------------------------------------------------------------
    void setDN(const char* dn);

    //----------------------------------------------------------------
    // Add a new (name, value) pair to the Item definition.
    //----------------------------------------------------------------
    void addAttribute(const char* name, const char* value);

    //----------------------------------------------------------------
    // Specify if an item with the same distinguished name must,
    // or must not, exist at the time that a DirectoryAgent attempts
    // to store this item.
    // By default, the update is forced through.
    //----------------------------------------------------------------
    void setItemMustExist(bool f = true);
    void setItemMustNotExist(bool f = true);

    //----------------------------------------------------------------
    // Get the distinguished name of this item.
    //----------------------------------------------------------------
    const char* dn() const;

    //----------------------------------------------------------------
    // Get the number of attributes (name,value) pairs which are
    // associated with this item.
    //----------------------------------------------------------------
    int attributes() const;

    //----------------------------------------------------------------
    // Get a name of the n'th attribute (numbered from 0).
    //----------------------------------------------------------------
    const char* attributeName(int n) const;

    //----------------------------------------------------------------
    // Get a value of the n'th attribute (numbered from 0).
    //----------------------------------------------------------------
    const char* attributeValue(int n) const;

    //----------------------------------------------------------------
    // Get a value of the n'th attribute matching the given name.
    //----------------------------------------------------------------
    const char* attributeValue(const char* name, int n = 0) const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaDirectoryItem();

    //----------------------------------------------------------------
    // Constructor creating the object using an existing
    // implementation instance.
    //----------------------------------------------------------------
    RomaDirectoryItem(RomaImplDirItem*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplDirItem* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaDirectoryAgent class.
//
// An application creates a RomaDirectoryAgent object when it wants
// to search or process directory items.
//--------------------------------------------------------------------
class KXCODLL RomaDirectoryAgent : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructor sets up search conditions. Defaults to using the
    // top-level Directory base rather than ROMA_BASE.
    //----------------------------------------------------------------
    RomaDirectoryAgent(RomaClient& client,
                       const char* startingPoint,
                       const char* searchName,
                       Roma::SearchType searchType,
                       Roma::SearchOption option = Roma::USE_NO_BASE);

    RomaDirectoryAgent(RomaBusinessElement& element,
                       const char* startingPoint,
                       const char* searchName,
                       Roma::SearchType searchType,
                       Roma::SearchOption option = Roma::USE_NO_BASE);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaDirectoryAgent();

    //----------------------------------------------------------------
    // The following are provided for compatibility with the ActiveX
    // API, but it is recommended that a separate RomaDirectoryAgent
    // is created for each set of search criteria.
    //----------------------------------------------------------------
    bool lookup(RomaClient& client,
                const char* startingPoint,
                const char* searchName,
                Roma::SearchType searchType,
                Roma::SearchOption option = Roma::USE_NO_BASE);

    bool lookup(RomaBusinessElement& element,
                const char* startingPoint,
                const char* searchName,
                Roma::SearchType searchType,
                Roma::SearchOption option = Roma::USE_NO_BASE);


    //----------------------------------------------------------------
    // METHODS FOR RETRIEVING DIRECTORY ITEMS.
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Get the number of items matching the selected criteria.
    // Optionally, return an indicator to say if the directory was
    // modified since the last lookup.
    //----------------------------------------------------------------
    int matches();
    int matches(bool& modified);

    //-----------------------------------------------------------------
    // Get the n'th item which matches the selected criteria and
    // return an indicator to say if the directory was modified since
    // the last lookup. n is counted from 0.
    //
    // The RomaDirectoryItem returned contains a snapshot of the whole
    // directory item.  Individual attributes can then be retrieved
    // via methods on the RomaDirectoryItem.
    //
    // The application owns the RomaDirectoryItem and is responsible
    // for freeing it.
    //-----------------------------------------------------------------
    RomaDirectoryItem* item(int n);
    RomaDirectoryItem* item(int n, bool& modified);

    //-----------------------------------------------------------------
    // When an application calls processAll() the action() method will
    // be invoked on all Items which match the specified criteria, 
    // or until a call to action returns false.
    //
    // processAll() returns the number of items processed.
    //-----------------------------------------------------------------
    int processAll();


    //-----------------------------------------------------------------
    // METHODS FOR MODIFYING THE DIRECTORY.
    //-----------------------------------------------------------------

    //-----------------------------------------------------------------
    // Remove item defined by distinguished name. Id and password
    // are required if directory access credential was not specified to
    // the RomaClient::connect() or the RomaBusinessElement::connect().
    //
    // Returns true if the item was successfully removed.
    //-----------------------------------------------------------------
    bool deleteItem(const char* dn);
    bool deleteItem(const char* dn, const char* id, const char* pw);

    //-----------------------------------------------------------------
    // Store the supplied Item in the Directory. Id and password
    // are required if directory access credential was not specified to
    // the RomaClient::connect() or the RomaBusinessElement::connect().
    //
    // Returns true if the update is successful.
    //-----------------------------------------------------------------
    bool storeItem(RomaDirectoryItem& i);
    bool storeItem(RomaDirectoryItem& i, 
                   const char* id, 
                   const char* pw);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //-----------------------------------------------------------------
    // Override this method to implement some specific processing on a
    // RomaDirectoryItem.
    //-----------------------------------------------------------------
    virtual bool action(RomaDirectoryItem* item);

    //-----------------------------------------------------------------
    // Default constructor.
    //-----------------------------------------------------------------
    RomaDirectoryAgent();

    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplDirAgent* m_impl;

private:
    //-----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //-----------------------------------------------------------------
    RomaDirectoryAgent(const RomaDirectoryAgent&);
    RomaDirectoryAgent& operator = (const RomaDirectoryAgent&);

    friend class RomaImplProxy;
};


#endif // __KXCORDA_HPP

