//=========================================================================
//
// Module      : cppems.cpp
//
// Description : Sample Roma C++ EMS Server program
//
//=========================================================================
//
// Copyright Candle Corporation 2003
//
//=========================================================================
//
// Disclaimer:  This sample program is provided for
//              information purposes only.  Candle neither makes
//              any warranties or representations regarding this
//              software, or its use.  This sample program is
//              provided "as is" and Candle expressly disclaims the
//              warranties of merchantability and fitness for a
//              particular purpose.
//
//=========================================================================
//
// Description:
//    Sample Roma C++ EMS Server Program.
//
//    This program shows a very simple Roma EMS server in action.
//    It is intended to show representative usage of the Roma
//    C++ EMS API.
//
// Details:
//    This program reads EMS messages from its input queue, and
//    processes them according to command-line options.
//    It can browse and display the contents of the EMS messages,
//    or it can modify the user-message data and return the message
//    back onto the queue on which the reported operation failed.
//
//    So, for example, if a Roma server application fails to GET
//    a message from its input queue (for whatever reason), and
//    Error Profiling is configured to cause an EMS message to be
//    sent to this EMS server, then the EMS server can modify the
//    message data and return the message back to the Roma server's
//    input queue to allow it to re-attempt the GET operation.
//
//    If a PUT operation to a specific queue fails, and an EMS
//    message is set to this EMS server, then it will again replace
//    the message data and attempt to PUT the message onto the
//    original target queue again.
//
// Dependencies:
//    Requires the relevant services to be set up in the directory
//    and the underlying transport queues to be configured in
//    support of the routes defined for the directory entry.
//    Error Profiling needs to be configured to have EMS as the
//    required action, and the Busines Service representing this
//    EMS server to be the EMSServer that is used.
//
// Invocation Syntax:
// cppems [ -e ElementName ] [ -r ] [ -v ] [ -? ]
//
// Input Parameters (all optional):
//    -e ElementName
//       Specifies the Roma Business Element name that this EMS
//       Server will initialize as.  Default is "TestElementEMS".
//
//    -r
//       By default the server peeks messages from the EMS queue
//       and prints detailed information on them. With this option,
//       the server will get messages from the EMS queue, modify
//       their data and put back onto the queue around which the
//       original error occurred.
//
//    -v
//       Turns on "verbose" mode.  Default is off.
//
//    -? Prints usage information and exits.
//
//
// Return Value:
//    One of the following values:
//       EXIT_SUCCESS -- Successful completion.
//       EXIT_FAILURE -- Unsuccessful completion.
//
//=========================================================================
// $Id: kxcosems.cpp,v 1.4 2004/02/06 18:44:42 e123479 Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosems.cpp,v $
//=========================================================================


// System headers.
#include <stdio.h>
#include <time.h>

// The Message Exchange API header.
#include <kxcoall.hpp>


//=====================================================================
// Simple exception class.
//=====================================================================

class RomaSampleException
{
public:
    RomaSampleException(const char* s)
    { 
        size_t len = strlen(s) + 1;
        m_str = new char[len];
        memcpy(m_str, s, len);
    }

    RomaSampleException(const RomaSampleException& e) : m_str(e.m_str)
    {
        ((RomaSampleException&)e).m_str = NULL;
    }

    RomaSampleException& operator=(const RomaSampleException& e)
    {
        if (this != &e)
        {
            m_str = e.m_str;
            ((RomaSampleException&)e).m_str = NULL;
        }

        return *this;
    }

    ~RomaSampleException() 
    { 
        delete m_str; 
    }

    const char* getText() const
    {
        return m_str;
    }

private:
    char*       m_str;
};


//=====================================================================
// Maximal attempts to return failed message to the addressee.
//=====================================================================
#define RETRY_MAX   5


//=====================================================================
// Roma error checking macro.
//=====================================================================
#define CHECK_ERROR(c) \
    if (c.anyError()) throw c.error();


//=====================================================================
// Forward declarations of helper routines.
//=====================================================================
static void runServer();
static void returnMsg(RomaBusinessElement& elem,
                      const RomaMessageInfo& info, 
                      const RomaBeRequest& msg);
static void printMsgInfo(RomaBusinessElement& elem,
                         const RomaMessageInfo& info,
                         const RomaBeRequest& msg);
static void printTheme(const RomaThemeInfo& ti);
static void printMsg(const RomaBeRequest& msg);
static void printWorkflow(const RomaWorkflow& wf);
static void printExcData(const RomaExceptionInfo& ei);
static const char* trailerType2Str(Roma::ILSType type);
static const char* appType2Str(Roma::AppType type);
static const char* envType2Str(Roma::EnvType type);
static const char* envOs2Str(Roma::EnvOS type);
static const char* callType2Str(int type);
static void printAsHex(const void* data, size_t length);
static void parseOptions(char** argv);
static void usage();


//=====================================================================
// EMS Server name.
//=====================================================================
const char* s_name = "TestElementEMS";

//=====================================================================
// Sample verbosity indicator.
//=====================================================================
bool s_verbose = false;

//=====================================================================
// Type of operation.
//=====================================================================
bool s_display = true;


//=====================================================================
// Main routine.
//=====================================================================

int main(int argc, char* argv[])
{
    try
    {
        parseOptions(++argv);
        runServer();
    }
    catch (const RomaSampleException& e)
    {
        // Handle sample exception

        cerr << "EMS Server failed\n";
        cerr << e.getText() << endl;
        return EXIT_FAILURE;
    }
    catch (const RomaError& r)
    {
        // Handle Roma exception
        cerr << "EMS Server failed\n";

        const char* text = r.errorText();
        cerr << "Primary:\n";
        if (text != NULL) cerr << text;

        if (r.status() == Roma::STATUS_BOTH)
        {
            cerr << "\nSecondary:\n";
            text = r.errorText(Roma::SUBCLASS_SECONDARY);
            if (text != NULL) cerr << text;
        }

        cerr << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}


//=====================================================================
// Helper routines.
//=====================================================================


//=====================================================================
// Message retrieving and processing routine.
//=====================================================================
void runServer()
{
    //-----------------------------------------------------------------
    // Setup GET options.
    //-----------------------------------------------------------------

    RomaGetOptions opt;

    // Accept native messages.
    opt.allowNative(true);

    // Prohibit EAR invokation.
    opt.setNoEms(true);

    // Close the recieved message theme.
    opt.setCloseTheme(true);

    if (s_display)
    {
        // Leave messages on the queue.
        opt.setOperation(Roma::PEEK_FIRST);

        // Don't wait for a new message.
        opt.setWaitInterval(0);
    }

    if (s_verbose)
    {
        cout << "\nInitializing EMS Server as " << s_name;
        cout << "\nAction is " << (s_display ? "DISPLAY" : "RETURN");
        cout << endl;
    }

    //-----------------------------------------------------------------
    // Create and connect the server.
    //-----------------------------------------------------------------

    RomaBusinessElement elem(s_name);
    CHECK_ERROR(elem);

    if (s_display)
    {
        cout << "\nRoma Exception Message Display\n";
    }

    //-----------------------------------------------------------------
    // Message processing loop.
    //-----------------------------------------------------------------

    while (true)
    {
        //-------------------------------------------------------------
        // Receive a message.
        //-------------------------------------------------------------

        RomaBeRequest* msg = elem.get(opt);

        if (msg == NULL)
        {
            CHECK_ERROR(elem);

            if (s_verbose)
            {
                cout << "No more EMS messages\n";
            }

            break;
        }

        if (s_verbose || s_display)
        {
            cout << "\nMessage received. Processing...\n";
            cout.flush();
        }

        //-------------------------------------------------------------
        // Extract detailed message information.
        //-------------------------------------------------------------

        RomaMessageInfo info;

        if (!elem.interpretMessage(info, *msg, Roma::INTMSG_THEME))
        {
            CHECK_ERROR(elem);
        }

        if (s_display)
        {
            //---------------------------------------------------------
            // Print message information and proceed with the next one.
            //---------------------------------------------------------

            printMsgInfo(elem, info, *msg);

            if (info.hasTheme())
            {
                // It is application responsibility to close
                // the extracted theme.

                if (!elem.closeTheme(info.getTheme()))
                {
                    CHECK_ERROR(elem);
                }
            }

            cout << endl;
            opt.setOperation(Roma::PEEK_NEXT);
        }
        else
        {
            //---------------------------------------------------------
            // Modify message data and put the message onto the queue
            // around which the original error occurred.
            // returnMsg() closes the extracted theme.
            //---------------------------------------------------------

            if (s_verbose)
            {
                cout << "Returning EMS message ...\n";
                cout.flush();
            }

            returnMsg(elem, info, *msg);
        }

        delete msg;
    }
}

//=====================================================================
// Updates the theme workflow and returns the message to the sender.
//=====================================================================
void returnMsg(RomaBusinessElement& elem,
               const RomaMessageInfo& info, 
               const RomaBeRequest& msg)
{
    if (!info.hasTheme())
    {
        throw RomaSampleException("No theme in the message information");            
    }

    //-----------------------------------------------------------------
    // Extract theme details.
    //-----------------------------------------------------------------

    RomaThemeInfo ti;
    RomaTheme theme = info.getTheme();

    if (!elem.queryTheme(ti, theme))
    {
        CHECK_ERROR(elem);
    }

    //-----------------------------------------------------------------
    // Check the retry count to prevent message bouncing.
    //-----------------------------------------------------------------

    if (ti.getRetryCount() >= RETRY_MAX)
    {
        throw RomaSampleException("Retry count exceeded maximum");
    }

    //-----------------------------------------------------------------
    // Check workflow route steps.
    //-----------------------------------------------------------------

    RomaWorkflow wf = ti.getWorkflow();
    RomaRouteList routes = wf.getRoutes();

    if (routes.isEmpty())
    {
        throw RomaSampleException("No routes in the workflow");
    }

    //-----------------------------------------------------------------
    // Remove bridges from the route. The message will be sent to
    // the first non-bridge application.
    //-----------------------------------------------------------------

    RomaRoute wr;
    RomaRouteListIterator it = routes.getIterator();
    bool rebuild = false;

    while (it.getNext(wr) && wr.getType() == Roma::BRIDGE)
    {
        rebuild = true;
        routes.remove(wr);
    }

    if (routes.isEmpty())
    {
        throw RomaSampleException(
            "No routes in the workflow after bridges removal");
    }

    //-----------------------------------------------------------------
    // Rebuild the workflow to put the message back to the
    // sender's input queue.
    //-----------------------------------------------------------------

    if (info.hasException())
    {
        RomaExceptionInfo ei = info.getException();

        if (ei.hasErrorData())
        {
            RomaExcErrorData ed = ei.getErrorData();
            unsigned long ct = ed.getCallType();

            if (ct == Roma::CALLTYPE_GETREQUEST ||
                ct == Roma::CALLTYPE_GETRESPONSE)
            {
                RomaConfig rc = ei.getRomaConfig();
                wf.getWorkflowService().setComponent(
                    rc.getComponentName());

                RomaRoute wr;
                wr.setType(rc.getElementType());
                wr.setTargetQueue(rc.getQueueName());
                wr.setTargetElement(rc.getElementName());
                wr.setTargetComponent(rc.getComponentName());
                routes.insert(wr);
                rebuild = true;
            }
        }
    }

    //-----------------------------------------------------------------
    // If the workflow route is modified, we need to build a new
    // theme that will be used for the message return. The old
    // theme should be closed.
    //-----------------------------------------------------------------

    if (rebuild)
    {
        if (s_verbose)
        {
            cout << "Rebuilding the theme ...\n";
            cout.flush();
        }

        if (!elem.buildTheme(theme, theme, ti,
            Roma::THEMEINFO_WORKFLOW, true))
        {
            CHECK_ERROR(elem);
        }
    }

    if (s_verbose)
    {
        RomaRoute wr;
        routes.getAt(wr, 0);
        cout << "Returning message to the application ";
        cout << wr.getTargetElement();
        cout << " on the queue ";
        cout << wr.getTargetQueue() << endl;
        cout.flush();
    }

    //-----------------------------------------------------------------
    // Create the response message that will be put back onto the
    // queue around which the original error occurred.
    //-----------------------------------------------------------------

    // The constructor will set the response with the
    // message ID, correlation ID and theme of the original message.
    RomaBeRequest req = info.hasMessage() ? info.getBeRequest() : msg;
    RomaBeResponse rsp(&req);

    // Set the new theme.
    rsp.setTheme(theme);

    // Use existing message and correlation IDs.
    rsp.setSelector(Roma::USE_MSGID_AND_CORRELID);

    // Set the new message data.
    char mdata[] = "MESSAGE-MODIFIED-BY-EMS";
    rsp.setMsgBuffer(mdata, sizeof(mdata) - 1);

    // Enable the retry counter.
    rsp.setIncrementRetryCount(true);

    // Close the theme when the message is sent.
    rsp.setCloseTheme(true);

    // Don't invoke error analysis services.
    rsp.setNoEms(true);

    // Don't put in a transactional queue.
    rsp.setNoSyncAllowed(true);

    // The server doesn't need a notfication.
    rsp.setNoNotification(true);

    // Prohibit a message transformation.
    rsp.setNoTransformation(true);

    //-----------------------------------------------------------------
    // Send the modified message.
    //-----------------------------------------------------------------

    if (!elem.put(rsp))
    {
        CHECK_ERROR(elem);
    }
}


//=====================================================================
// Prints message information.
//=====================================================================
void printMsgInfo(RomaBusinessElement& elem,
                  const RomaMessageInfo& info, 
                  const RomaBeRequest& msg)
{
    //-----------------------------------------------------------------
    // If there is exception information, print it.
    //-----------------------------------------------------------------

    if (info.hasException())
    {
            printExcData(info.getException());
    }
    else
    {
        cout << "\n No exception information\n";
    }

    //-----------------------------------------------------------------
    // If there is workflow information, print it.
    //-----------------------------------------------------------------

    if (info.hasWorkflow())
    {
        printWorkflow(info.getWorkflow());
    }
    else
    {
        cout << "\n No Workflow information\n";
    }

    //-----------------------------------------------------------------
    // If there is message data, print it.
    // Otherwise, print the received message.
    //-----------------------------------------------------------------

    printMsg(info.hasMessage() ? info.getBeRequest() : msg);

    //-----------------------------------------------------------------
    // If there is a theme, print its details.
    //-----------------------------------------------------------------

    if (info.hasTheme())
    {
        RomaTheme theme = info.getTheme();

        if (theme.isNull())
        {
            cout << "\n Theme is NULL\n";
            return;
        }

        RomaThemeInfo ti;

        if (!elem.queryTheme(ti, theme))
        {
            CHECK_ERROR(elem);
        }

        printTheme(ti);
    }
    else
    {
        cout << "\n No Theme Information\n";
    }
}


//=====================================================================
// Prints theme details.
//=====================================================================
void printTheme(const RomaThemeInfo& ti)
{
    cout << "\n Theme Information :";
    cout << "\n  Retry Count : " << ti.getRetryCount();
    cout << "\n  Theme ID ...: ";
    printAsHex((const void*)ti.getThemeId(), Roma::ID_LENGTH);

    //-----------------------------------------------------------------
    // Print confirmation flags.
    //-----------------------------------------------------------------

    if ((ti.getConfirmFlags() & Roma::CONFIRM_DELIVERY) != 0)
    {
        cout << "  Confirmation of Delivery is enabled\n";
    }

    if ((ti.getConfirmFlags() & Roma::CONFIRM_ARRIVAL) != 0)
    {
        cout << "  Confirmation of Arrival is enabled\n";
    }

    //-----------------------------------------------------------------
    // Print persistent trailers.
    //-----------------------------------------------------------------

    RomaTrailerList trailers = ti.getTrailers();
    if (trailers.isEmpty()) return;

    cout << "  Persistent Trailers :\n";

    int i = 1;
    RomaTrailer rt;
    RomaTrailerListIterator it = trailers.getIterator();

    while (it.getNext(rt))
    {
        cout << "   Trailer #" << i++ << " :";
        cout << "\n    Type ..: " << trailerType2Str(rt.getType());
        cout << "\n    Method : " << rt.getMethod();
        cout << "\n    Data ..:";

        unsigned long dlen;
        const void* data = rt.getData(dlen);
        printAsHex(data, dlen);
    }          
}


//=====================================================================
// Prints message data.
//=====================================================================
void printMsg(const RomaBeRequest& msg)
{
    cout << "\n Message Information :";
    cout << "\n  Format .......: " << msg.format();
    cout << "\n  Type .........: " << msg.msgType();
    cout << "\n  Persistence ..: " << msg.persistence();
    cout << "\n  Priority .....: " << msg.priority();
    cout << "\n  Feedback .....: " << msg.feedback();
    cout << "\n  Expiry .......: " << msg.expiry();
    cout << "\n  Code Page ....: " << msg.codepage();
    cout << "\n  Encoding .....: " << msg.encoding();
    cout << "\n  Status .......: " << msg.status();
    cout << "\n  Backout Count : " << msg.backoutCount();
    cout << "\n  Format Name ..: " << msg.formatName();
    cout << "\n  Message Name .: " << msg.msgName();

    time_t ts = msg.timestamp().seconds();
    cout << "\n  Timestamp ....: " << ctime(&ts) << endl;

    cout << "  Message ID ...: ";
    printAsHex((const void*)msg.msgId(), Roma::ID_LENGTH);
    cout << "  Correl ID ....: ";
    printAsHex((const void*)msg.correlId(), Roma::ID_LENGTH);
    cout << "  Tag Data .....: ";
    printAsHex(msg.tagBuffer(), msg.tagDataLength());
    cout << "  Message Data .: ";
    printAsHex(msg.msgBuffer(), msg.msgDataLength());
}


//=====================================================================
// Prints workflow information.
//=====================================================================
void printWorkflow(const RomaWorkflow& wf)
{
    //-----------------------------------------------------------------
    // Print workflow service data.
    //-----------------------------------------------------------------

    RomaWorkflowService ws = wf.getWorkflowService();

    cout << "\n Workflow Information :";
    cout << "\n  WorkFlow Service information:";
    cout << "\n   Current Service .....: " << ws.getService();
    cout << "\n   Current Component ...: " << ws.getComponent();
    cout << "\n   Audit ...............: " << ws.getAudit();
    cout << "\n   Source Service ......: " << ws.getSourceService();
    cout << "\n   Source Business Flow : " << ws.getSourceFlow();
    cout << "\n   Source Operation ....: " << ws.getSourceOperation();
    cout << endl;

    //-----------------------------------------------------------------
    // Print workflow route steps.
    //-----------------------------------------------------------------

    RomaRouteList routes = wf.getRoutes();

    if (routes.isEmpty())
    {
        cout << "  No Route Information\n";
        return;
    }

    cout << "  Route Information :\n";

    int i = 1;
    RomaRoute wr;
    RomaRouteListIterator it = routes.getIterator();

    while (it.getNext(wr))
    {
        cout << "   Route #" << i++ << " :";
        cout << "\n    Type : " << appType2Str(wr.getType());
        cout << "\n    Flags : " << wr.getFlags();
        cout << "\n    Target Queue : " << wr.getTargetQueue();
        cout << "\n    Target Element : " << wr.getTargetElement();
        cout << "\n    Target Component : " << wr.getTargetComponent();
        cout << "\n    Target Code Page : " << wr.getTargetCodePage();
        cout << "\n    Source Service : " << wr.getSourceService();
        cout << "\n    Source Flow : " << wr.getSourceFlow();
        cout << "\n    Source Operation : " << wr.getSourceOperation();
        cout << endl;
    }
}

//=====================================================================
// Prints exception data.
//=====================================================================
void printExcData(const RomaExceptionInfo& ei)
{
    cout << "\n Exception Information:\n";

    //-----------------------------------------------------------------
    // If there is Roma configuration data, print it.
    //-----------------------------------------------------------------

    if (ei.hasRomaConfig())
    {
        RomaConfig rc = ei.getRomaConfig();

        cout << "  Roma Configuration Data :";
        cout << "\n   Application Type .............: ";
        cout << appType2Str(rc.getElementType());
        cout << "\n   Application Name .............: ";
        cout << rc.getElementName();
        cout << "\n   Input Queue Name .............: ";
        cout << rc.getQueueName();
        cout << "\n   Associated Business Component : ";
        cout << rc.getComponentName();
        cout << "\n   Associated Business Service ..: ";
        cout << rc.getServiceName();
        cout << "\n   Node Name ....................: ";
        cout << rc.getNodeName() << endl;
    }
    else
    {
        cout << "  No Roma configuration information\n";
    }

    //-----------------------------------------------------------------
    // If there is system configuration data, print it.
    //-----------------------------------------------------------------

    if (ei.hasSystemConfig())
    {
        RomaSystemConfig sc = ei.getSystemConfig();

        cout << "  System Configuration Data :";
        cout << "\n   Environment Type : " << envType2Str(sc.getEnvType());
        cout << "\n   Environment OS ..: " << envOs2Str(sc.getEnvOS());
        cout << "\n   Domain Name .....: " << sc.getDomainName();
        cout << "\n   User Id .........: " << sc.getUserId();
        cout << "\n   Program Name ....: " << sc.getProgramName();
        cout << "\n   Process Id ......: " << sc.getProcessId();
        cout << "\n   Thread Id .......: " << sc.getThreadId() << endl;
    }
    else
    {
        cout << "  No system configuration information\n";
    }

    //-----------------------------------------------------------------
    // If there is a timestamp, print it.
    //-----------------------------------------------------------------

    if (ei.hasTimestamp())
    {
        time_t ts = ei.getTimestamp().seconds();
        cout << "  Timestamp : " << ctime(&ts) << endl;
    }
    else
    {
        cout << "  No timestamp information\n";
    }

    //-----------------------------------------------------------------
    // If there is error information, print it.
    //-----------------------------------------------------------------

    if (ei.hasErrorData())
    {
        RomaExcErrorData ed = ei.getErrorData();
        const RomaError& error = ed.getError();
        const char* text = error.errorText();

        cout << "  Error Data :";
        cout << "\n   Primary ...: ";
        if (text != NULL) cout << text;
        cout << "\n   Secondary .: ";
        text = error.errorText(Roma::SUBCLASS_SECONDARY);
        if (text != NULL) cout << text;
        cout << "\n   Call Type .: " << callType2Str(ed.getCallType());
        cout << "\n   Queue Name : " << ed.getQueueName() << endl;
    }
    else
    {
        cout << "  No error information\n";
    }

    //-----------------------------------------------------------------
    // If there is user data, print it.
    //-----------------------------------------------------------------

    if (ei.hasUserData())
    {
        cout << "  User Data: ";
        unsigned long dlen = 0;
        const void* data = ei.getUserData(dlen);
        printAsHex(data, dlen);
    }
    else
    {
        cout << "  No user data\n";
    }
}

//=====================================================================
// Returns string representation of the trailer type.
//=====================================================================
const char* trailerType2Str(Roma::ILSType type)
{
    switch (type)
    {
        case Roma::ILS_TYPE_XPORT:
            return "XPORT";
        case Roma::ILS_TYPE_USER:
            return "USER";
        case Roma::ILS_TYPE_ROUTER:
            return "ROUTER";
        case Roma::ILS_TYPE_SYSMGMT:
            return "SYSMGMT";
        case Roma::ILS_TYPE_ERROR:
            return "ERROR";
        case Roma::ILS_TYPE_XML:
            return "XML";
        case Roma::ILS_TYPE_XPTDATA:
            return "XPTDATA";
    }

    return "Unknown";
}

//=====================================================================
// Returns string representation of the application type.
//=====================================================================
const char* appType2Str(Roma::AppType type)
{
    switch (type)
    {
        case Roma::CLIENT:
            return "Client";
        case Roma::SERVER:
            return "Server";
        case Roma::BRIDGE:
            return "Bridge";
        case Roma::NATIVE_CLIENT:
            return "Native client";
        case Roma::NATIVE_SERVER:
            return "Native server";
    }

    return "Unknown";
}

//=====================================================================
// Returns string representation of the environment type.
//=====================================================================
const char* envType2Str(Roma::EnvType type)
{
    switch (type)
    {
        case Roma::ENVTYPE_INTEL:
            return "Intel";
        case Roma::ENVTYPE_MVS:
            return "MVS";
        case Roma::ENVTYPE_UNIX:
            return "Unix";
        case Roma::ENVTYPE_OS400:
            return "OS 400";
        case Roma::ENVTYPE_VMS:
            return "VMS";
    }

    return "Unknown";
}

//=====================================================================
// Returns string representation of the OS type.
//=====================================================================
const char* envOs2Str(Roma::EnvOS type)
{
    switch (type)
    {
        case Roma::ENVOS_AIX:
            return "AIX";
        case Roma::ENVOS_HPUX102:
            return "HP-UX 10";
        case Roma::ENVOS_HPUX11:
            return "HP-UX 11";
        case Roma::ENVOS_DUX:
            return "Digital Unix";
        case Roma::ENVOS_MVSBATCH:
            return "MVS Batch";
        case Roma::ENVOS_MVSCICS:
            return "CICS";
        case Roma::ENVOS_SUN:
            return "Solaris";
        case Roma::ENVOS_WIN95:
            return "Windows 95";
        case Roma::ENVOS_WINNT:
            return "Windows NT";
        case Roma::ENVOS_OS400:
            return "OS 400";
    }

    return "Unknown";
}

//=====================================================================
// Returns string representation of the call type.
//=====================================================================
const char* callType2Str(int type)
{
    switch (type)
    {
        case Roma::CALLTYPE_APPLICATION:
            return "APPLICATION";
        case Roma::CALLTYPE_PUTREQUEST:
            return "PUTREQUEST";
        case Roma::CALLTYPE_GETREQUEST:
            return "GETREQUEST";
        case Roma::CALLTYPE_PUTRESPONSE:
            return "PUTRESPONSE";
        case Roma::CALLTYPE_GETRESPONSE:
            return "GETRESPONSE";
        case Roma::CALLTYPE_BRIDGEPUT:
            return "BRIDGEPUT";
        case Roma::CALLTYPE_BRIDGEGET:
            return "BRIDGEGET";
        case Roma::CALLTYPE_PUTEXCEPTION:
            return "PUTEXCEPTION";
        case Roma::CALLTYPE_INITIALIZE:
            return "INITIALIZE";
        case Roma::CALLTYPE_TERMINATE:
            return "TERMINATE";
        case Roma::CALLTYPE_COMMIT:
            return "COMMIT";
        case Roma::CALLTYPE_ROLLBACK:
            return "ROLLBACK";
        case Roma::CALLTYPE_FREEBUFFER:
            return "FREEBUFFER";
        case Roma::CALLTYPE_CLOSEHANDLE:
            return "CLOSEHANDLE";
        case Roma::CALLTYPE_INTERPRETMSG:
            return "INTERPRETMSG";
        case Roma::CALLTYPE_WRITELOG:
            return "WRITELOG";
        case Roma::CALLTYPE_SENDALERT:
            return "SENDALERT";
        case Roma::CALLTYPE_OPENSTORAGEQ:
            return "OPENSTORAGEQ";
        case Roma::CALLTYPE_CLOSESTORAGEQ:
            return "CLOSESTORAGEQ";
        case Roma::CALLTYPE_DELETESTORAGEQ:
            return "DELETESTORAGEQ";
        case Roma::CALLTYPE_PUTTOSTORAGEQ:
            return "PUTTOSTORAGEQ";
        case Roma::CALLTYPE_GETFROMSTORAGEQ:
            return "GETFROMSTORAGEQ";
        case Roma::CALLTYPE_QUERYTHEME:
            return "QUERYTHEME";
        case Roma::CALLTYPE_BUILDTHEME:
            return "BUILDTHEME";
        case Roma::CALLTYPE_PURGEQTOEMS:
            return "PURGEQTOEMS";
        case Roma::CALLTYPE_REPORTERROR:
            return "REPORTERROR";
        case Roma::CALLTYPE_QUERYCONTEXT:
            return "QUERYCONTEXT";
        case Roma::CALLTYPE_SETLDAPSERVICE:
            return "SETLDAPSERVICE";
        case Roma::CALLTYPE_SETILSDATA:
            return "SETILSDATA";
        case Roma::CALLTYPE_GETILSDATA:
            return "GETILSDATA";
        case Roma::CALLTYPE_DIRLOOKUP:
            return "DIRLOOKUP";
        case Roma::CALLTYPE_DIRGETDN:
            return "DIRGETDN";
        case Roma::CALLTYPE_DIRUPDATEITEM:
            return "DIRUPDATEITEM";
        case Roma::CALLTYPE_DIRDELETEITEM:
            return "DIRDELETEITEM";
        case Roma::CALLTYPE_DIRGETMODCOUNT:
            return "DIRGETMODCOUNT";
    }

    return "Unknown";
}

//=====================================================================
// Prints a hexadecimal value of each byte of the data.
//=====================================================================
void printAsHex(const void* data, size_t length)
{
    if (data != NULL)
    {
        char s[10];
        const unsigned char* d = (const unsigned char*)data;

        for (size_t i = 0; i < length; ++i)
        {
            sprintf(s, "%02X", d[i]);
            cout << s;
        }
    }

    cout << endl;
}

//=====================================================================
// Parses command line options.
//=====================================================================
void parseOptions(char** argv)
{
    for (; *argv != NULL; ++argv)
    {
        const char* opt = *argv;

        if (*opt != '-' || opt[2])
        {
            ostrstream s;
            s << "Unknown option " << opt << ends;
            throw RomaSampleException(s.str());
        }

        switch (opt[1])
        {
            case 'e':
                ++argv;

                if (*argv == NULL)
                {
                    throw RomaSampleException(
                        "No element name after -e option");
                }

                s_name = *argv;
                break;

            case '?':
                usage();
                exit(EXIT_SUCCESS);

            case 'v':
                s_verbose = true;
                break;

            case 'r':
                s_display = false;
                break;
                                        
            default:
            {
                ostrstream s;
                s << "Unknown option " << opt << ends;
                throw RomaSampleException(s.str());
            }
        }
    }
}

//=====================================================================
// Prints usage information.
//=====================================================================
void usage()
{
    cout << "\nUsage: cppems [-e ElementName] [-r] [-v]\n"

            "\nOptions:\n\n"

            "\t-e ElementName\n"
            "\t\tSpecifies the Roma Business Element name that this EMS\n"
            "\t\tServer will initialize as. Default is TestElementEMS.\n\n"

            "\t-r\n"
            "\t\tBy default the server peeks messages from the EMS queue\n"
            "\t\tand prints detailed information on them. With this option,\n"
            "\t\tthe server will get messages from the EMS queue, modify\n"
            "\t\ttheir data and put back onto the queue around which the\n"
            "\t\toriginal error occurred.\n\n"

            "\t-v\n"
            "\t\tTurns on verbose mode. Default is off.\n\n";
}

